﻿-- TODO - požadavek3 ???

--! @file
--!
--! @brief              Modul IoDiagRegisters
--! @par "Projekt:"     FPGA Z102
--! @copyright          Copyright &copy; ZAT a.s. Všechna práva vyhrazena
--! @author             Petr Wolmut
--! @par "Požadavek"    ???
--! @par "SVN adresa:"  https://vyvoj.zat.lan/svn/ARM/trunk/Z102/FPGA_VHDL
--! @par "Popis modulu" Modul entity IoDiagRegisters, Definice standardního rozhraní pro detailní diagnostiku I/O
--!

--! Standardní knihovna
library IEEE;
--! Standardní knihovní balík
use IEEE.STD_LOGIC_1164.all;

--! Balík společných definicí pro FPGA Z10X (@ref PackageZ102)
use work.PackageZ102.all;

--! @brief <b> Definice standardního rozhraní pro detailní diagnostiku I/O </b>
--! @details
--! Blok @ref IoDiagRegisters zajišťuje standardní rozhraní pro diagnostiku vstupů a výstupů v systémech Z10X.
--! Zahrnuje standardní registry pro sumární a detailní diagnostiku.
--! Pro každý I/O kanál jsou vyhrazeny BYTEy příznaků detailní diagnostiky  (@ref DiagFlags_in a @ref DiagFlags_out) a masky (@ref DiagFlagsMasks_in),
--! dále pak bity pro povolení (@ref EnableDiag_in), společné mazání příznaků (@ref DiagCommonReset_in)
--! a sumární diagnostický výstup (@ref DiagSummary_out).
--! @warning
--! Počet I/O kanálů je určován z rozměru připojeného vstupního pole BYTE @ref DiagFlags_in,
--! rozměry ostatních vektorů a polí musí korespondovat s tímto rozměrem.
--!
entity IoDiagRegisters is
  generic(
    --! @brief Povolení maskování detailní diagnostiky
    --! @details
    --! Tento parametr má vliv na výstupní pole BYTE detailní diagnostiky @ref DiagFlags_out.
    --! Pokud je maskování zakázáno, kopírují se do výstupního pole přímo příznaky ze vstupního pole @ref DiagFlags_in.
    --! Při povoleném maskování jsou výstupní bity detailní diagnostiky @ref DiagFlags_out
    --! maskovány odpovídajícími bity v poli masek diagnostiky @ref DiagFlagsMasks_in.
    --!
    MaskDetailDiags_g : boolean
  );
  port(
    --! Asynchronní reset
    Reset_in           : in  std_logic;
    --! Hodinový signál
    Clock_in           : in  std_logic;
    --! Vektor povolení diagnostiky pro jednotlivé I/O kanály
    EnableDiag_in      : in  std_logic_vector;
    --! Vstupní pole diagnostických BYTE pro jednotlivé kanály, hlavní vstupní signál z HW
    DiagFlags_in       : in  tStdLogicByteArray;
    --! Vstupní pole masek diagnostiky pro jednotlivé kanály
    DiagFlagsMasks_in  : in  tStdLogicByteArray;
    --! Vstupní vektor RESETů diagnostiky
    DiagCommonReset_in : in  std_logic_vector;
    --! Výstupní pole diagnostických BYTE pro jednotlivé kanály (maskované v závislosti na @ref MaskDetailDiags_g)
    DiagFlags_out      : out tStdLogicByteArray;
    --! Výstupní vektor sumární diagnostiky pro jednotlivé kanály
    DiagSummary_out    : out std_logic_vector
  );
end IoDiagRegisters;

--! @brief <b> Definice standardního rozhraní pro detailní diagnostiku I/O - implementace </b>
--! @details
--! Architekturu bloku tvoří registry pro uchování příznaků diagnostiky @ref DetailDiagnostics
--! a obslužný proces, který řeší zápis dat do těchto registrů, maskování, generování sumární diagnostiky
--! a případné mazání diagnostických příznaků jednotlivých I/O kanálů.
--!
architecture IoDiagRegisters_A of IoDiagRegisters is

  --! Pole BYTE s detailní diagnostikou
  signal DetailDiagnostics : tStdLogicByteArray(DiagFlags_in'range);
  --! Pole BYTE výstupních signálů detailní diagnostiky
  signal DiagFlags_out_sig : tStdLogicByteArray(DiagFlags_in'range);

begin
  -- přiřazení výstupů
  DiagFlags_out <= DiagFlags_out_sig;

  --! @brief Obsluha diagnostiky
  --! @param[in]  EnableDiag_in      Povolení jednotlivých kanálů
  --! @param[in]  DiagFlags_in       Detailní diagnostika (vstup)
  --! @param[in]  DiagFlagsMasks_in  Masky diagnostiky
  --! @param[in]  DiagCommonReset_in Požadavky na mazání (RESET) jednotlivých kanálů
  --! @param[out] DiagSummary_out    Sumární diagnostika
  --! @param[out] DetailDiagnostics  Detailní diagnostika (výstup)
  --! @details
  --! Proces zpracovává vstupní příznaky detailní diagnostiky pro jednotlivé I/O kanály.
  --! Pro každý I/O kanál je vyhrazen jeden BYTE diagnostických příznaků (@ref DiagFlags_in), jejichž význam je určen pro jednotlivé typy I/O
  --! v konstantách standardních diagnostických struktur (např. @ref BASIC_BI_DIAG_BITS v balíku @ref Package_IoSpace).
  --!
  --! Pomocí masek pro jednotlivé I/O kanály (@ref DiagFlagsMasks_in) je možno jednotlivé příznaky blokovat.
  --! Blokování konkrétního příznaku zajišťuje logická hodnota "0" v příslušném bitu masky.
  --! Diagnostické příznaky jsou kopírovány do výstupní detailní diagnostiky (@ref DetailDiagnostics)
  --! v závislosti na nastavení generického parametru @ref MaskDetailDiags_g.
  --! Pokud je maskování detailní diagnostiky povoleno, kopírují se na výstup maskované příznaky, v opačném případě přímo vstupní příznaky.
  --! Maskované příznaky lze také mazat (pokud je vstupní příznak neaktivní - vstup má prioritu) pomocí vektoru bitů @ref DiagCommonReset_in.
  --! Pro každý I/O kanál (BYTE diagnostických příznaků) je vyhrazen jeden bit pro mazání příznaků celého kanálu.
  --! @note
  --! Globální maskování diagnostiky pro jednotlivé (celé) I/O kanály umožňuje vektor povolovacích bitů @ref EnableDiag_in.
  --! @details
  --! Maskovaná detailní diagnostika je dále zpracovávána do sumární diagnostiky @ref DiagSummary_out,
  --! pro každý I/O kanál je vyhrazen jeden bit sumární diagnostiky.
  --! Sumární diagnostický bit je aktivní, pokud je alespoň jeden bit z BYTEu detailní diagnostiky po maskování aktivní.
  --!
  --! @vhdlflow Vývojový diagram
  DiagData : process(Reset_in, Clock_in)
  begin
    if Reset_in = '1' then
      for Ix in EnableDiag_in'range loop
        DiagSummary_out(Ix) <= '0';
      end loop;
      for Ix in DetailDiagnostics'range loop
        DetailDiagnostics(Ix) <= (others => '0');
      end loop;
    elsif Clock_in'event and (Clock_in = '1') then
      -- diagnostická data, pro všechny výstupy
      for Ix in DiagFlags_in'range loop
        if EnableDiag_in(Ix) = '1' then -- povolený výstup
          DiagSummary_out(Ix) <= '0';   -- prepare zero
          for Iy in DiagFlags_in(Ix)'range loop
            if DetailDiagnostics(Ix)(Iy) = '1' then -- jeden z diagnostických příznaků aktivní
              DiagSummary_out(Ix) <= '1';
            end if;
          end loop;
        else                            -- zakázaný výstup
          DiagSummary_out(Ix) <= '0';
        end if;
      end loop;

      -- diagnostické bity, pro všechny výstupy
      for Ix in DiagFlags_in'range loop
        for Iy in DiagFlags_in(Ix)'range loop
          if (DiagFlags_in(Ix)(Iy) and DiagFlagsMasks_in(Ix)(Iy)) = '1' then -- příznak aktivní a nemaskovaný
            DetailDiagnostics(Ix)(Iy) <= '1';
          elsif DiagCommonReset_in(Ix) = '1' then -- reset
            DetailDiagnostics(Ix)(Iy) <= '0';
          end if;
        end loop;
      end loop;

      -- výstupy detailní diagnostiky
      if MaskDetailDiags_g then
        -- detailní diagnostika maskována
        DiagFlags_out_sig <= DetailDiagnostics;
      else
        -- detailní diagnostika přímo z příznaků
        DiagFlags_out_sig <= DiagFlags_in;
      end if;

    end if;
  end process DiagData;

end IoDiagRegisters_A;

